﻿//EX1001_ADC_EDK
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//


using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.Threading;
using Microsoft.SPOT.Hardware.SJJ;
using Microsoft.SPOT.Hardware.SJJ.SJJ_ADC_Driver;
using System.IO.Ports;

namespace EX1001_ADC_EDK
{
    public class Program
    {
        public static void Main()
        {
            Debug.Print(Resources.GetString(Resources.StringResources.String1));
            App myApp = new App();
            myApp.Run();
        }

        public class App
        {

            // Create ADC driver object
            SJJ_ADC_Driver myADCDriver = new SJJ_ADC_Driver();

            // Create green LED object
            OutputPort myGreenLED;

            public void Run()
            {
                // Open the ADC device
                SerialPort myAdcHandle = myADCDriver.ADC_Open();

                // Create green LED object for visual feedback and turn it off to start
                myGreenLED = new OutputPort(Pins.GREEN_LED, false);

                int iADCSourceIdx = 0;
                Int16 i16ConvertSample;
                SJJ_ADC_Driver.ADC_Status eStatus;

                // Set ADC clock divider to /4
                eStatus = myADCDriver.ADC_SetClkDiv(SJJ_ADC_Driver.ADC_ClkDiv.eADCDivide4, myAdcHandle);
                if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                {
                    Debug.Print("ADC_SetClkDiv failed: " + eStatus.ToString());
                }

                // Start ADC Testing
                // Sample all analog input channels
                while (true)
                {
                    // Turn off ADC
                    eStatus = myADCDriver.ADC_Off(myAdcHandle);
                    if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                    {
                        Debug.Print("ADC_Off failed: " + eStatus.ToString());
                    }

                    // Set new ADC source
                    switch (iADCSourceIdx)
                    {
                        case 0:
                            eStatus = myADCDriver.ADC_SetChannel(SJJ_ADC_Driver.ADC_Source.eAnalogChannel0, myAdcHandle);
                            if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                            {
                                Debug.Print("ADC_SetChannel eAnalogChannel0 failed: " + eStatus.ToString());
                            }
                            break;
                        case 1:
                            eStatus = myADCDriver.ADC_SetChannel(SJJ_ADC_Driver.ADC_Source.eAnalogChannel1, myAdcHandle);
                            if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                            {
                                Debug.Print("ADC_SetChannel eAnalogChannel1 failed: " + eStatus.ToString());
                            }
                            break;
                        case 2:
                            eStatus = myADCDriver.ADC_SetChannel(SJJ_ADC_Driver.ADC_Source.eAnalogChannel2, myAdcHandle);
                            if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                            {
                                Debug.Print("ADC_SetChannel eAnalogChannel2 failed: " + eStatus.ToString());
                            }
                            break;
                        case 3:
                            eStatus = myADCDriver.ADC_SetChannel(SJJ_ADC_Driver.ADC_Source.eAnalogChannel3, myAdcHandle);
                            if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                            {
                                Debug.Print("ADC_SetChannel eAnalogChannel3 failed: " + eStatus.ToString());
                            }
                            break;
                        case 4:
                            eStatus = myADCDriver.ADC_SetChannel(SJJ_ADC_Driver.ADC_Source.eAnalogChannel4, myAdcHandle);
                            if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                            {
                                Debug.Print("ADC_SetChannel eAnalogChannel4 failed: " + eStatus.ToString());
                            }
                            break;
                        default:
                            eStatus = myADCDriver.ADC_SetChannel(SJJ_ADC_Driver.ADC_Source.eBatteryVoltage, myAdcHandle);
                            if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                            {
                                Debug.Print("ADC_SetChannel eBatteryVoltage failed: " + eStatus.ToString());
                            }
                            break;
                    }

                    Debug.Print("Source Selected");
                    // Turn on ADC
                    eStatus = myADCDriver.ADC_On(myAdcHandle);
                    if (eStatus != SJJ_ADC_Driver.ADC_Status.eStatusWriteSuccess)
                    {
                        Debug.Print("ADC_On failed");
                    }
                    // Toggle green LED on at beginning of ADC sample
                    if ((iADCSourceIdx % 2) == 0)
                    {
                        // Toggle green LED on
                        myGreenLED.Write(true);
                    }
                    else
                    {
                        // Toggle green LED off
                        myGreenLED.Write(false);
                    }

                    Debug.Print("ADC On");

                    // Read and output ADC
                    i16ConvertSample = (Int16)myADCDriver.ADC_Read(myAdcHandle);
                    if ((i16ConvertSample == SJJ_ADC_Driver.ADC_COM_ERROR) || (i16ConvertSample == SJJ_ADC_Driver.ADC_CONVERSION_ERROR))
                    {
                        Debug.Print("ADC_Read failed: " + i16ConvertSample.ToString());
                    }
                    else
                    {
                        Debug.Print("ADC Read for Ch " + iADCSourceIdx.ToString() + " = " + i16ConvertSample.ToString());
                    }

                    // Increment and wrap index
                    if (++iADCSourceIdx > 5)
                    {
                        iADCSourceIdx = 0;
                    }

                    Thread.Sleep(500);
                }
            }
        }
    }
}
