//
//DesktopClient
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using Socket = System.Net.Sockets.Socket;
using System.Threading;

namespace DesktopClient
{
    public partial class DesktopClient : Form
    {
        // Delegates for thread safe accessing of form controls
        delegate void SetStatusTextBoxCallback(string sText);
        delegate string GetStatusTextBoxCallback();
        delegate string GetPWM1ValueCallback();
        delegate string GetPWM2ValueCallback();
        delegate string GetPWM3ValueCallback();
        delegate void SetTextBox_PWM1FreqCallback(string sText);
        delegate void SetTextBox_PWM2DutyCycleCallback(string sText);
        delegate void SetTextBox_PWM3DutyCycleCallback(string sText);
        delegate void SetLEDButtonStateCallback(bool bLEDState);

        private bool bLEDState = false;
        private bool bLEDStateRequest = false;
        private bool bPWM1StateChange = false;
        private bool bPWM2StateChange = false;
        private bool bPWM3StateChange = false;
        private bool bCloseStateChange = false;
        static IPAddress IPAddr;
        static Int32 iPort = 12000;
        static Socket SandRSocket;

        //String message constants
        const int c_MinPWMFreq = 1000;
        const int c_FiveSeconds = 5000000;
        const string sMsgLEDOn = "LED ON";
        const string sMsgLEDOff = "LED OFF";
        const string sMsgHdrPWM1 = "PWM1 ";
        const string sMsgTrailerPWM1 = " Hz";
        const string sMsgHdrPWM2 = "PWM2 ";
        const string sMsgTrailerPWM2 = " %";
        const string sMsgHdrPWM3 = "PWM3 ";
        const string sMsgTrailerPWM3 = " %";
        const string sMsgClose = "Close";

        public DesktopClient()
        {
            InitializeComponent();

            // Put current version in form title
            this.Text = this.ProductName + "  V" + this.ProductVersion;

            // Initialize controls
            // Clear PWM text boxes
            textBox_PWM1Freq.Text = "";
            textBox_PWM2DutyCycle.Text = "";
            textBox_PWM3DutyCycle.Text = "";

            // Initialize trackbar controls
            trackBar_PWM1Freq.Minimum = 0;   // Control minimum will be zero, but minimum frequency is 1KHz
            trackBar_PWM1Freq.Maximum = 100000; // Maximum frequency is 100KHz
            trackBar_PWM2DutyCycle.Minimum = 0; // Minimum duty cycles are 0 %
            trackBar_PWM3DutyCycle.Minimum = 0;
            trackBar_PWM2DutyCycle.Maximum = 100; // Maximum duty cycles are 100%
            trackBar_PWM3DutyCycle.Maximum = 100;
            trackBar_PWM1Freq.Value = 10000; // Set initial frequency to be 10KHz
            trackBar_PWM2DutyCycle.Value = 50; // Set initial duty cycles mid-range
            trackBar_PWM3DutyCycle.Value = 50;
            trackBar_PWM1Freq.TickFrequency = (trackBar_PWM1Freq.Maximum - trackBar_PWM1Freq.Minimum) / 10;
            trackBar_PWM2DutyCycle.TickFrequency = (trackBar_PWM2DutyCycle.Maximum - trackBar_PWM2DutyCycle.Minimum) / 10;
            trackBar_PWM3DutyCycle.TickFrequency = (trackBar_PWM3DutyCycle.Maximum - trackBar_PWM3DutyCycle.Minimum) / 10;
            trackBar_PWM1Freq.LargeChange = trackBar_PWM1Freq.TickFrequency;
            trackBar_PWM2DutyCycle.LargeChange = trackBar_PWM2DutyCycle.TickFrequency;
            trackBar_PWM3DutyCycle.LargeChange = trackBar_PWM3DutyCycle.TickFrequency;
            trackBar_PWM1Freq.SmallChange = (trackBar_PWM1Freq.Maximum - trackBar_PWM1Freq.Minimum) / 100;
            trackBar_PWM2DutyCycle.SmallChange = (trackBar_PWM2DutyCycle.Maximum - trackBar_PWM2DutyCycle.Minimum) / 100;
            trackBar_PWM3DutyCycle.SmallChange = (trackBar_PWM3DutyCycle.Maximum - trackBar_PWM3DutyCycle.Minimum) / 100;

            // Disable controls until socket connection
            DisableControls();

            // Prompt for IP address entry
            textBox_Status.Text = "Enter IP address";
        }

        private void DisableControls()
        {
            // Disable controls until socket connection is made
            button_ServerClose.Enabled = false;
            buttonGreenLED.Enabled = false;
            bLEDState = false;
            trackBar_PWM1Freq.Enabled = false;
            trackBar_PWM2DutyCycle.Enabled = false;
            trackBar_PWM3DutyCycle.Enabled = false;
        }

        private void EnableControls()
        {
            // Enable controls until socket connection is made
            button_ServerClose.Enabled = true;
            buttonGreenLED.Enabled = true;
            if (bLEDState)
            {
                buttonGreenLED.BackColor = Color.Green;
            }
            else
            {
                buttonGreenLED.BackColor = Color.Transparent;
            }
            trackBar_PWM1Freq.Enabled = true;
            trackBar_PWM2DutyCycle.Enabled = true;
            trackBar_PWM3DutyCycle.Enabled = true;
        }

        private void ClearPWMTextBoxes()
        {
            SetTextBox_PWM1Freq("");
            SetTextBox_PWM2DutyCycle("");
            SetTextBox_PWM3DutyCycle("");
        }

        private void SetStatusTextBox(string sText)
        {
            // Thread safe methode for setting status text
            if (this.textBox_Status.InvokeRequired)
            {
                SetStatusTextBoxCallback d = new SetStatusTextBoxCallback(SetStatusTextBox);
                this.Invoke(d, new object[] { sText });
            }
            else
            {
                this.textBox_Status.Text = sText;
            }
        }

        private string GetStatusTextBox()
        {
            string sReturnText;

            // thread safe method for getting the text string
            if (this.textBox_Status.InvokeRequired)
            {
                GetStatusTextBoxCallback d = new GetStatusTextBoxCallback(GetStatusTextBox);
                sReturnText = (string) this.Invoke(d, new object[] { });
            }
            else
            {
                sReturnText = this.textBox_Status.Text;
            }
            return (sReturnText);
        }

        private string GetPWM1Value()
        {
            string sReturnText;

            // thread safe method for getting the trackbar value
            if (this.trackBar_PWM1Freq.InvokeRequired)
            {
                GetPWM1ValueCallback d = new GetPWM1ValueCallback(GetPWM1Value);
                sReturnText = (string)this.Invoke(d, new object[] { });
            }
            else
            {
                sReturnText = this.trackBar_PWM1Freq.Value.ToString();
            }

            return (sReturnText);
        }

        private string GetPWM2Value()
        {
            string sReturnText;

            // thread safe method for getting the trackbar value
            if (this.trackBar_PWM2DutyCycle.InvokeRequired)
            {
                GetPWM2ValueCallback d = new GetPWM2ValueCallback(GetPWM2Value);
                sReturnText = (string)this.Invoke(d, new object[] { });
            }
            else
            {
                sReturnText = this.trackBar_PWM2DutyCycle.Value.ToString();
            }

            return (sReturnText);
        }

        private string GetPWM3Value()
        {
            string sReturnText;

            // thread safe method for getting the trackbar value
            if (this.trackBar_PWM3DutyCycle.InvokeRequired)
            {
                GetPWM3ValueCallback d = new GetPWM3ValueCallback(GetPWM3Value);
                sReturnText = (string)this.Invoke(d, new object[] { });
            }
            else
            {
                sReturnText = this.trackBar_PWM3DutyCycle.Value.ToString();
            }

            return (sReturnText);
        }

        private void SetTextBox_PWM1Freq(string sText)
        {
            // thread safe method for setting the textbox
            if (this.textBox_PWM1Freq.InvokeRequired)
            {
                SetTextBox_PWM1FreqCallback d = new SetTextBox_PWM1FreqCallback(SetTextBox_PWM1Freq);
                this.Invoke(d, new object[] { sText });
            }
            else
            {
                this.textBox_PWM1Freq.Text = sText;
            }
        }

        private void SetTextBox_PWM2DutyCycle(string sText)
        {
            // thread safe method for setting the textbox
            if (this.textBox_PWM2DutyCycle.InvokeRequired)
            {
                SetTextBox_PWM2DutyCycleCallback d = new SetTextBox_PWM2DutyCycleCallback(SetTextBox_PWM2DutyCycle);
                this.Invoke(d, new object[] { sText });
            }
            else
            {
                this.textBox_PWM2DutyCycle.Text = sText;
            }
        }

        private void SetTextBox_PWM3DutyCycle(string sText)
        {
            // thread safe method for setting the textbox
            if (this.textBox_PWM3DutyCycle.InvokeRequired)
            {
                SetTextBox_PWM3DutyCycleCallback d = new SetTextBox_PWM3DutyCycleCallback(SetTextBox_PWM3DutyCycle);
                this.Invoke(d, new object[] { sText });
            }
            else
            {
                this.textBox_PWM3DutyCycle.Text = sText;
            }
        }

        private void SetLEDButtonState(bool bLEDState)
        {
            // thread safe methode for setting button state
            if (this.buttonGreenLED.InvokeRequired)
            {
                SetLEDButtonStateCallback d = new SetLEDButtonStateCallback(SetLEDButtonState);
                this.Invoke(d, new object[] { bLEDState });
            }
            else
            {
                if (bLEDState)
                {
                    this.buttonGreenLED.BackColor = Color.Green;
                }
                else
                {
                    this.buttonGreenLED.BackColor = Color.Transparent;
                }
            }
        }

        private void OnPWM1TrackBarChange()
        {
            SetStatusTextBox("Sending PWM1 TrackBar command");
            bPWM1StateChange = true;
        }

        private void OnPWM2TrackBarChange()
        {
            SetStatusTextBox("Sending PWM2 TrackBar command");
            bPWM2StateChange = true;
        }

        private void OnPWM3TrackBarChange()
        {
            SetStatusTextBox("Sending PWM3 TrackBar command");
            bPWM3StateChange = true;
        }

        private void DesktopClient_Load(object sender, EventArgs e)
        {

        }

        private void button_ServerConnect_click(object sender, EventArgs e)
        {
            
            // Add Socket connect code
            // Connect to port
            try
            {
                iPort = Convert.ToInt32(textBox_ServerSocket.Text);
                IPAddr = IPAddress.Parse(textBox_ServerIP.Text);

                // Create socket and connect to the server's IP address and port
                SandRSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                SandRSocket.Connect(new IPEndPoint(IPAddr, iPort));

                // Start background worker thread
                this.ReceiverBackgroundWorkerThread.RunWorkerAsync();

                SetStatusTextBox("Connected to:  " + IPAddr.ToString() + " on port: " + iPort.ToString());
                // Disable connect button
                button_ServerConnect.Enabled = false;

                // Clear PWM TextBoxes
                ClearPWMTextBoxes();

                // Enable controls
                EnableControls();
            }
            catch (Exception eX)
            {
                SetStatusTextBox(eX.Message);
            }
        }

        private void button_ServerClose_click(object sender, EventArgs e)
        {
            SetStatusTextBox("Sending socket close message");

            // Enable connect button
            button_ServerConnect.Enabled = true;

            // Disable controls
            DisableControls();

            bCloseStateChange = true;
        }

        private void ReceiverBackgroundWorkerThread_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker myBW = sender as BackgroundWorker;

            Byte[] bytesReceived = new Byte[256];
            int iNumBytesReceived;

            // Main thread activity loop
            try
            {

                while (!e.Cancel)
                {
                    // Check LED command status
                    if (bLEDStateRequest != bLEDState)
                    {
                        // LED state has command pending for state change
                        if (bLEDStateRequest)
                        {
                            Byte[] bytesToSend = Encoding.UTF8.GetBytes(sMsgLEDOn);
                            SandRSocket.Send(bytesToSend);
                        }
                        else
                        {
                            Byte[] bytesToSend = Encoding.UTF8.GetBytes(sMsgLEDOff);
                            SandRSocket.Send(bytesToSend);
                        }
                    }
                    else if (bPWM1StateChange)
                    {
                        // Reset state variable
                        bPWM1StateChange = false;

                        // Build message and send
                        Byte[] bytesToSend = Encoding.UTF8.GetBytes(sMsgHdrPWM1 + GetPWM1Value() + sMsgTrailerPWM1);
                        SetStatusTextBox("Sending message: " + sMsgHdrPWM1 + GetPWM1Value() + sMsgTrailerPWM1);
                        SandRSocket.Send(bytesToSend);
                    }
                    else if (bPWM2StateChange)
                    {
                        // Reset state variable
                        bPWM2StateChange = false;

                        // Build message and send
                        Byte[] bytesToSend = Encoding.UTF8.GetBytes(sMsgHdrPWM2 + GetPWM2Value() + sMsgTrailerPWM2);
                        SandRSocket.Send(bytesToSend);
                    }
                    else if (bPWM3StateChange)
                    {
                        // Reset state variable
                        bPWM3StateChange = false;

                        // Build message and send
                        Byte[] bytesToSend = Encoding.UTF8.GetBytes(sMsgHdrPWM3 + GetPWM3Value() + sMsgTrailerPWM3);
                        SandRSocket.Send(bytesToSend);
                    }
                    else if (bCloseStateChange)
                    {
                        // Reset state variable
                        bCloseStateChange = false;

                        // Build message and send
                        byte[] bytesToSend = Encoding.UTF8.GetBytes(sMsgClose);
                        SandRSocket.Send(bytesToSend);
                    }

                    // Check for server response
                    // Do blocking receive with timeout for loopback
                    if (SandRSocket.Poll(c_FiveSeconds, SelectMode.SelectRead))
                    {
                        iNumBytesReceived = SandRSocket.Receive(bytesReceived);
                    }
                    else
                    {
                        iNumBytesReceived = 0;
                        SetStatusTextBox("No bytes received from server");
                    }

                    if (iNumBytesReceived > 0)
                    {
                        // Parse response and take action
                        string sMsgReceived = new string(UTF8Encoding.UTF8.GetChars(bytesReceived, 0, iNumBytesReceived));
                        SetStatusTextBox("Received[" + iNumBytesReceived.ToString() + "][" + sMsgReceived.Length.ToString() + "]: " + sMsgReceived);

                        if (sMsgReceived.Contains(sMsgLEDOn))
                        {
                            // LED state was turned on
                            bLEDState = true;
                            SetLEDButtonState(bLEDState);
                        }
                        else if (sMsgReceived.Contains(sMsgLEDOff))
                        {
                            // LED state was turn off
                            bLEDState = false;
                            SetLEDButtonState(bLEDState);
                        }
                        else if (sMsgReceived.Contains(sMsgHdrPWM1))
                        {
                            // PWM1 command acknowledged, update textbox
                            int iSubStringIdx = sMsgHdrPWM1.Length;
                            string sDisplayString = sMsgReceived.Substring(iSubStringIdx);
                            SetTextBox_PWM1Freq(sDisplayString);
                        }
                        else if (sMsgReceived.Contains(sMsgHdrPWM2))
                        {
                            // PWM2 command acknowledged, update textbox
                            int iSubStringIdx = sMsgHdrPWM2.Length;
                            string sDisplayString = sMsgReceived.Substring(iSubStringIdx);
                            SetTextBox_PWM2DutyCycle(sDisplayString);
                        }
                        else if (sMsgReceived.Contains(sMsgHdrPWM3))
                        {
                            // PWM3 command acknowledged, update textbox
                            int iSubStringIdx = sMsgHdrPWM3.Length;
                            string sDisplayString = sMsgReceived.Substring(iSubStringIdx);
                            SetTextBox_PWM3DutyCycle(sDisplayString);
                        }
                        else if (sMsgReceived.Contains(sMsgClose))
                        {
                            // Socket close cammand acknowledged, close socket
                            this.ReceiverBackgroundWorkerThread.CancelAsync();
                        }
                        else
                        {
                            // Unknown response
                            SetStatusTextBox("Unknown server response: " + GetStatusTextBox());
                        }
                    }

                    if (myBW.CancellationPending)
                    {
                        e.Cancel = true;
                    }
                }
            }
            catch (Exception eXception)
            {
                SetStatusTextBox("Thread Exception: " + eXception.Message);
            }
            finally
            {
                // Close port
                try
                {
                    e.Cancel = true;

                    SandRSocket.Close();
                    SetStatusTextBox("Socket closed");

                }
                catch (Exception eX)
                {
                    SetStatusTextBox(eX.Message);
                }
            }
        }

        private void ReceiverBackgroundWorkerThread_Completed(object sender, RunWorkerCompletedEventArgs e)
        {
            // Thread exit cleanup
            SetStatusTextBox(GetStatusTextBox() + "...Worker thread exited");
        }

        private void trackBar_PWM1Freq_MouseUp(object sender, MouseEventArgs e)
        {
            OnPWM1TrackBarChange();
        }

        private void trackBar_PWM1Freq_KeyUp(object sender, KeyEventArgs e)
        {
            OnPWM1TrackBarChange();
        }

        private void trackBar_PWM2DutyCycle_KeyUp(object sender, KeyEventArgs e)
        {
            OnPWM2TrackBarChange();
        }

        private void trackBar_PWM2DutyCycle_MouseUp(object sender, MouseEventArgs e)
        {
            OnPWM2TrackBarChange();
        }

        private void trackBar_PWM3DutyCycle_KeyUp(object sender, KeyEventArgs e)
        {
            OnPWM3TrackBarChange();
        }

        private void trackBar_PWM3DutyCycle_MouseUp(object sender, MouseEventArgs e)
        {
            OnPWM3TrackBarChange();
        }

        private void buttonGreenLED_click(object sender, EventArgs e)
        {
            // Send LED on/off message

            if (bLEDState)
            {
                bLEDStateRequest = false;
            }
            else
            {
                bLEDStateRequest = true;
            }

            if (bLEDStateRequest)
            {
                SetStatusTextBox("Sending command: " + sMsgLEDOn);
            }
            else
            {
                SetStatusTextBox("Sending command: " + sMsgLEDOff);
            }
        }

    }
}