﻿//EX1103_LAN_EDK
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using Microsoft.SPOT;
using System.Threading;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Hardware.SJJ;
using Microsoft.SPOT.Hardware.SJJ.SJJ_PWM_Driver;
using Microsoft.SPOT.Net;
using Microsoft.SPOT.Net.NetworkInformation;
using System.Net;
using System.Net.Sockets;
using Socket = System.Net.Sockets.Socket;
using System.Text;
using System.IO.Ports;

namespace EX1103_LAN_EDK
{
    public class Program
    {
        public const string c_VersionNumber = "4.0.1.3";

        public static void Main()
        {
            // Sign on
            Debug.Print("MF Socket Server V" + c_VersionNumber);

            App myApp = new App();

            // rerun server if it exits
            while (true)
            {
                myApp.Run();

                Debug.Print("Restarting Server");
            }
        }

    }


    public class App
    {
        const Int32 iPort = 12000;
        const int c_WaitInfinite = -1;
        const int c_FiveSeconds = 5000000;
        const int iReceiveBufferSize = 256;
        const uint c_MinPWM1 = 225; //Hz
        const uint c_uiConvertError = 999999999;
        const uint c_uiMaxPWMFreq = 100000; //100 KHz
        const int c_iInitDutyCycle = 50; // Initialize duty cycles to 50%
        bool bRunServer = true;
        int iBytesReceived = 0;
        int iBytesSent = 0;
        SJJ_PWM_Driver.PWM_Status eStatusPWM;

        // Message constants
        const string sMsgLEDOn = "LED ON";
        const string sMsgLEDOff = "LED OFF";
        const string sMsgHdrPWM1 = "PWM1";
        const string sMsgHdrPWM2 = "PWM2";
        const string sMsgHdrPWM3 = "PWM3";
        const string sMsgClose = "Close";

        // Control objects
        // PWM driver object
        SJJ_PWM_Driver myPWMDriver;
        bool bPWMState;

        // green LED object
        OutputPort myGreenLED;

        // Virtual serial port handle for PWM access
        SerialPort myPwmHandle;

        Socket server;

        public App()
        {
            // Create green LED GPIO and PWM objects for control accrss
            myGreenLED = new OutputPort(Pins.GREEN_LED, false);

            // Initialize PWM's and start with them off
            InitPWM(false);

            // Create a socket, bind it to the server's port and listen for client connections
            server = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            IPEndPoint localEndPoint = new IPEndPoint(IPAddress.Any, iPort);

            server.Bind(localEndPoint);
            server.Listen(Int32.MaxValue);
        }

        public void Run()
        {
            // Report IP Address
            NetworkInterface[] mynetwork = NetworkInterface.GetAllNetworkInterfaces();

            foreach (NetworkInterface mynetworkInterface in mynetwork)
            {
                Debug.Print("Local IP: " + mynetworkInterface.IPAddress);
                Debug.Print("Local MAC: " + ConvertByteToHexString(mynetworkInterface.PhysicalAddress[0]) + " " +
                                            ConvertByteToHexString(mynetworkInterface.PhysicalAddress[1]) + " " +
                                            ConvertByteToHexString(mynetworkInterface.PhysicalAddress[2]) + " " +
                                            ConvertByteToHexString(mynetworkInterface.PhysicalAddress[3]) + " " +
                                            ConvertByteToHexString(mynetworkInterface.PhysicalAddress[4]) + " " +
                                            ConvertByteToHexString(mynetworkInterface.PhysicalAddress[5]));
            }
            Debug.Print("Port: " + iPort.ToString());

            try
            {
                // Infinite wait for a connection before exiting and failing
                bRunServer = server.Poll(c_WaitInfinite, SelectMode.SelectRead);

                if (bRunServer)
                {
                    Debug.Print("Connection received");
                }
            }
            catch (Exception e)
            {
                Debug.Print("Socket Exception: " + e.ToString());

                bRunServer = false;
            }

            finally
            {
                // Wait for a client to connect
                Socket clientSocket = server.Accept();

                // 'using' ensures that the socket gets closed
                using (clientSocket)
                {

                    while (bRunServer)
                    {
                        {   //Begin scope of receive buffer
                            byte[] bReceiveBuffer = new byte[iReceiveBufferSize];

                            // Check for received data
                            if (clientSocket.Poll(c_FiveSeconds, SelectMode.SelectRead))
                            {
                                if (clientSocket.Available == 0)
                                {
                                    // We waited 5 seconds and no bytes are ready to read,
                                    iBytesReceived = 0;
                                }
                                else
                                {

                                    iBytesReceived = clientSocket.Receive(bReceiveBuffer, clientSocket.Available, SocketFlags.None);

                                    Debug.Print("Bytes received = " + iBytesReceived.ToString());

                                    // check for socket error
                                    if (iBytesReceived < 0)
                                    {
                                        bRunServer = false;
                                    }
                                }
                            }
                            else
                            {
                                iBytesReceived = 0;
                            }

                            // loopback received data
                            if (iBytesReceived > 0)
                            {
                                // Parse the command received
                                // Convert byte message to string for parsing
                                string sMsgReceived = new string(UTF8Encoding.UTF8.GetChars(bReceiveBuffer));

                                Debug.Print("Message received: " + sMsgReceived);

                                // Find substrings
                                string[] sMsgSubs = sMsgReceived.Split(null);

                                if (string.Equals(sMsgReceived, sMsgLEDOn))
                                {
                                    // Turn on green LED
                                    myGreenLED.Write(true);
                                }
                                else if (string.Equals(sMsgReceived, sMsgLEDOff))
                                {
                                    // Turn off green LED
                                    myGreenLED.Write(false);
                                }
                                else if (string.Equals(sMsgSubs[0], sMsgHdrPWM1))
                                {
                                    // PWM1 command
                                    uint uiSetFreq = ConvertToUint(sMsgSubs[1]);

                                    if (uiSetFreq != c_uiConvertError)
                                    {
                                        // check for freq == 0 and turn it off
                                        if (uiSetFreq == 0)
                                        {
                                            // Turn off PWM1
                                            eStatusPWM = myPWMDriver.PWM_Off(myPwmHandle);
                                            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                            {
                                                Debug.Print("PWM_Off failed: " + eStatusPWM.ToString());
                                            }
                                            else
                                            {
                                                bPWMState = false;
                                            }
                                        }
                                        else if (0 < uiSetFreq && uiSetFreq <= c_MinPWM1)
                                        {
                                            // check for 0 < freq <= c_MinPWM1 and set freq == c_MinPWM1 & turn on
                                            eStatusPWM = myPWMDriver.PWM_SetFreq(c_MinPWM1, myPwmHandle);
                                            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                            {
                                                Debug.Print("PWM_SetFreq failed: " + eStatusPWM.ToString());
                                            }
                                            else if (!bPWMState)
                                            {
                                                // Turn on PWM
                                                eStatusPWM = myPWMDriver.PWM_On(myPwmHandle);
                                                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                                {
                                                    Debug.Print("PWM_On failed: " + eStatusPWM.ToString());
                                                }
                                                else
                                                {
                                                    bPWMState = true;
                                                }
                                            }
                                        }
                                        else
                                        {
                                            // else set freq == command freq and turn on
                                            eStatusPWM = myPWMDriver.PWM_SetFreq(uiSetFreq, myPwmHandle);
                                            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                            {
                                                Debug.Print("PWM_SetFreq failed: " + eStatusPWM.ToString());
                                            }
                                            else if (!bPWMState)
                                            {
                                                // Turn on PWM
                                                eStatusPWM = myPWMDriver.PWM_On(myPwmHandle);
                                                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                                {
                                                    Debug.Print("PWM_On failed: " + eStatusPWM.ToString());
                                                }
                                                else
                                                {
                                                    bPWMState = true;
                                                }
                                            }
                                        }

                                        // Reset duty cycle to 50%
                                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM1, 50, myPwmHandle);
                                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                        {
                                            Debug.Print("PWM_SetDutyCycle ePWM1 failed: " + eStatusPWM.ToString());
                                        }
                                    }
                                    else
                                    {
                                        Debug.Print("Frequency string conversion error");
                                    }
                                }
                                else if (string.Equals(sMsgSubs[0], sMsgHdrPWM2))
                                {
                                    // PWM2 command
                                    uint uiSetDutyCycle = ConvertToUint(sMsgSubs[1]);

                                    if (uiSetDutyCycle != c_uiConvertError)
                                    {
                                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM2_CPLD, (int)uiSetDutyCycle, myPwmHandle);
                                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                        {
                                            Debug.Print("PWM_SetDutyCycle ePWM2_CPLD failed: " + eStatusPWM.ToString());
                                        }
                                    }
                                    else
                                    {
                                        Debug.Print("Duty Cycle string conversion error");
                                    }
                                }
                                else if (string.Equals(sMsgSubs[0], sMsgHdrPWM3))
                                {
                                    // PWMe command
                                    uint uiSetDutyCycle = ConvertToUint(sMsgSubs[1]);

                                    if (uiSetDutyCycle != c_uiConvertError)
                                    {
                                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM3_CPLD, (int)uiSetDutyCycle, myPwmHandle);
                                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                                        {
                                            Debug.Print("PWM_SetDutyCycle ePWM3_CPLD failed: " + eStatusPWM.ToString());
                                        }
                                    }
                                    else
                                    {
                                        Debug.Print("Duty Cycle string conversion error");
                                    }
                                }
                                else if (string.Equals(sMsgReceived, sMsgClose))
                                {
                                    // Socket close
                                    bRunServer = false;
                                }
                                else
                                {
                                    // Unknown command
                                    Debug.Print("Unknown command: " + sMsgReceived);
                                }

                                // Echo back command for confirmation
                                if (iBytesReceived > 0)
                                {
                                    iBytesSent = clientSocket.Send(bReceiveBuffer, iBytesReceived, SocketFlags.None);

                                    //check for socket error
                                    if (iBytesSent < 0)
                                    {
                                        bRunServer = false;
                                    }

                                }
                            }
                        }   //End scope of receive buffer
                    }

                }

                Debug.Print("Server Exiting");
            }


        }

        public uint ConvertToUint(string sDecimalString)
        {
            uint uiReturnVal = c_uiConvertError;
            uint uiConvertVal = 0;

            // Convert string to byte array
            byte[] byDecimalArray = Encoding.UTF8.GetBytes(sDecimalString);

            // Note byte ordering for decimal strings converted to byte arrays.
            uint uiPwrTen = 1;
            int idx;
            for (idx = (byDecimalArray.Length - 1); idx >= 0; idx--)
            {
                if ((byDecimalArray[idx] >= 0x30) && (byDecimalArray[idx] <= 0x39))
                {
                    uiConvertVal += (uint)(byDecimalArray[idx] & 0x0F) * uiPwrTen;
                    uiPwrTen *= 10;
                }
                else
                {
                    // non-decimal character in string
                    break;
                }

            }

            //Check for error
            if (idx < 0)
            {
                //Conversion success
                uiReturnVal = uiConvertVal;
            }

            return (uiReturnVal);
        }

        public string ConvertByteToHexString(byte bByte)
        {
            byte[] bHexBytes = new byte[2];

            bHexBytes[0] = (byte)(bByte >> 4);
            bHexBytes[1] = (byte)(bByte & 0x0f);

            for (int i = 0; i < 2; i++)
            {
                if (bHexBytes[i] < 0xa)
                {
                    bHexBytes[i] |= 0x30;
                }
                else
                {
                    bHexBytes[i] = (byte)((bHexBytes[i] - 9) | 0x40);
                }
            }
            string sReturnString = new string(UTF8Encoding.UTF8.GetChars(bHexBytes));

            return (sReturnString);
        }

        public void InitPWM(bool bPWMOnOff)
        {
            // Create PWM object
            myPWMDriver = new SJJ_PWM_Driver();

            // Open the PWM device
            myPwmHandle = myPWMDriver.PWM_Open();

            // Turn PWM1 off
            eStatusPWM = myPWMDriver.PWM_Off(myPwmHandle);
            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
            {
                Debug.Print("PWM_Off failed: " + eStatusPWM.ToString());
            }

            // Set PWM1 frequency to maximum for this application
            eStatusPWM = myPWMDriver.PWM_SetFreq(c_uiMaxPWMFreq, myPwmHandle);
            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
            {
                Debug.Print("PWM_SetFreq failed: " + eStatusPWM.ToString());
            }

            // Set PWM1 duty cycle to 50%
            eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM1, c_iInitDutyCycle, myPwmHandle);
            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
            {
                Debug.Print("PWM_SetDutyCycle ePWM1 failed: " + eStatusPWM.ToString());
            }

            // Set CPLD PWM2 duty cycle to 50%
            eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM2_CPLD, c_iInitDutyCycle, myPwmHandle);
            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
            {
                Debug.Print("PWM_SetDutyCycle ePWM2_CPLD failed: " + eStatusPWM.ToString());
            }

            // Set CPLD PWM3 duty cycle to 50%
            eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM3_CPLD, c_iInitDutyCycle, myPwmHandle);
            if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
            {
                Debug.Print("PWM_SetDutyCycle ePWM3_CPLD failed: " + eStatusPWM.ToString());
            }

            // Turn on/off PWM
            if (bPWMOnOff)
            {
                eStatusPWM = myPWMDriver.PWM_On(myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_On failed: " + eStatusPWM.ToString());
                }
            }
            else
            {
                eStatusPWM = myPWMDriver.PWM_Off(myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_On failed: " + eStatusPWM.ToString());
                }
            }
        }
    }
}
