﻿//Button
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

//Code from a .NET Micro Framework SDK project.

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Presentation.Shapes;
using Microsoft.SPOT.Input;

namespace Microsoft.SPOT.Presentation.Button
{
    public class ButtonWPF : UIElement
    {
        /// <summary>
            /// Constructs a button with the specified caption and font, and 
            /// default size.
            /// </summary>
            /// <param name="caption"></param>
            /// <param name="font"></param>
            public ButtonWPF(string caption, Font font)
            {
                _caption = caption;
                _font = font;
                HorizontalAlignment = HorizontalAlignment.Left;
                VerticalAlignment = VerticalAlignment.Bottom;

                int textWidth;
                int textHeight;
                _font.ComputeExtent(_caption, out textWidth, out textHeight);

                _width = textWidth + _textMarginX * 2;
                _height = textHeight + _textMarginY * 2;
            }

            /// <summary>
            /// Constructs a button with the specified caption, font, and size.
            /// </summary>
            /// <param name="caption"></param>
            /// <param name="font"></param>
            /// <param name="width"></param>
            /// <param name="height"></param>
            public ButtonWPF(string caption, Font font, int width, int height)
            {
                _width = width;
                _height = height;
                _caption = caption;
                _font = font;
            }

            /// <summary>
            /// Overrides OnRender to do our own drawing.
            /// </summary>
            /// <param name="dc"></param>
            public override void OnRender(DrawingContext dc)
            {
                int x;
                int y;

                SolidColorBrush brush;
                Pen pen;
                Color color;
                Pen shade1;
                Pen shade2;

                // Check the pressed state, and draw accordingly.
                if (_pressed)
                {
                    brush = _pressedBackgroundBrush;
                    pen = _pressedBorderPen;
                    color = _pressedForeColor;
                    shade1 = _darkShade;
                    shade2 = _lightShade;
                }
                else
                {
                    brush = _normalBackgroundBrush;
                    pen = _borderPen;
                    color = _foreColor;
                    shade1 = _lightShade;
                    shade2 = _darkShade;
                }

                GetLayoutOffset(out x, out y);

                // Draw the base rectangle of the button
                dc.DrawRectangle(brush, pen, 1, 1, _width - 1, _height - 1);

                // Draw the caption
                string caption = _caption;
                dc.DrawText(ref caption, _font, color, 0, _textMarginY, _width, 
                    _height, _alignment, _trimming);

                // Shade the outline of the rectangle for classic button look
                dc.DrawLine(shade1, 1, 1, _width - 1, 1);
                dc.DrawLine(shade1, 1, 1, 1, _height - 1);
                dc.DrawLine(shade2, _width - 1, 1, _width - 1, _height - 1);
                dc.DrawLine(shade2, 1, _height - 1, _width - 1, _height - 1);
            }

            public event EventHandler Click;

            /// <summary>
            /// Handles the touch down event.
            /// </summary>
            /// <param name="e"></param>

            protected override void OnTouchDown(TouchEventArgs e)
            {
                // Flag for the drawing state.
                _pressed = true;

                TouchCapture.Capture(this);
                // Trigger a redraw.
                Invalidate();
            }

            /// <summary>
            /// Handles the touch up event.
            /// </summary>
            /// <param name="e"></param>
            protected override void OnTouchUp(TouchEventArgs e)
            {
                // Flag for the drawing state.
                _pressed = false;

                TouchCapture.Capture(this, CaptureMode.None);

                // Trigger a redraw.
                Invalidate();

                // Fire a click event.
                EventArgs args = new EventArgs();
                OnClick(args);
            }

            /// <summary>
            /// Handles the touch move.
            /// </summary>
            /// <param name="e"></param>

            protected override void OnTouchMove(TouchEventArgs e)
            {
            }

            /// <summary>
            /// Handles a click event.
            /// </summary>
            /// <param name="e"></param>
            protected virtual void OnClick(EventArgs e)
            {
                if (Click != null)
                {
                    Click(this, e);
                }
            }

            /// <summary>
            /// 
            /// </summary>
            /// <param name="arrangeWidth"></param>
            /// <param name="arrangeHeight"></param>
            protected override void ArrangeOverride(int arrangeWidth, 
                int arrangeHeight)
            {
            }

            /// <summary>
            /// 
            /// </summary>
            /// <param name="availableWidth"></param>
            /// <param name="availableHeight"></param>
            /// <param name="desiredWidth"></param>
            /// <param name="desiredHeight"></param>
            protected override void MeasureOverride(int availableWidth, 
                int availableHeight, out int desiredWidth, 
                out int desiredHeight)
            {
                desiredWidth = (availableWidth > _width) ? 
                _width : availableWidth;
                desiredHeight = (availableHeight > _height) ? 
                _height : availableHeight;
            }

            private SolidColorBrush _normalBackgroundBrush = 
                new SolidColorBrush(ColorUtility.ColorFromRGB(192, 192, 192));
            private SolidColorBrush _pressedBackgroundBrush = 
                new SolidColorBrush(ColorUtility.ColorFromRGB(128, 128, 128));

            private Pen _borderPen = 
                new Pen(ColorUtility.ColorFromRGB(128, 128, 128));

            private Pen _pressedBorderPen = 
                new Pen(ColorUtility.ColorFromRGB(128, 128, 128));

            private Pen _lightShade = 
                new Pen(ColorUtility.ColorFromRGB(216, 216, 216));
            private Pen _darkShade = 
                new Pen(ColorUtility.ColorFromRGB(64, 64, 64));

            int _width;
            int _height;
            string _caption = "";
            Font _font = null;
            Color _foreColor = ColorUtility.ColorFromRGB(0, 0, 0);
            Color _pressedForeColor = ColorUtility.ColorFromRGB(255, 255, 255);
            private TextTrimming _trimming = TextTrimming.WordEllipsis;
            private TextAlignment _alignment = TextAlignment.Center;
            protected int _textMarginX = 16;
            protected int _textMarginY = 8;
            protected bool _pressed = false;
        
    }
}
