﻿//Serializer Commands in a single library
//
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using System.Text;
using System.Threading;
using System.IO.Ports;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;


namespace SJJEMS.RobotSerializer
{
    public class RobotSerializer
    {

        
        protected static SerialPort COMPort; 
        protected static byte[] binCOMPortRead = new byte[16];

        public void RobotInitialize(String SerializerPort)
        {
            COMPort = new SerialPort(SerializerPort, (int)BaudRate.Baudrate19200, Parity.None, 8);
            COMPort.ReadTimeout = 10;
            COMPort.Open();
        }


        public void RobotMove(int Motor1, int Motor2)
        {
            //Create command string
            String MotorsGo = "mogo 1:" + Motor1.ToString() + " 2:" + Motor2.ToString() + "\r";
            byte[] boutMotorsGo = new byte[MotorsGo.Length];
            boutMotorsGo = Encoding.UTF8.GetBytes(MotorsGo);
            
            //Send command
            COMPort.Write(boutMotorsGo, 0, boutMotorsGo.Length);
            Thread.Sleep(100);

            //dummy read to clear buffer of ACK
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);            
        }

        public int RobotSensor(int SensorPort)
        {
            //Create command string
            String SensorGet = "sensor " + SensorPort.ToString() + "\r";
            byte[] boutSensorGet = new byte[SensorGet.Length];
            boutSensorGet = Encoding.UTF8.GetBytes(SensorGet);

            //Send Command
            COMPort.Write(boutSensorGet, 0, boutSensorGet.Length);
            Thread.Sleep(50);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);

            //Convert value to a number
            int SensorValue = StringToInt(binCOMPortRead);

            //Return final sensor value
            return SensorValue;
            
        }

        public void RobotServo(int ServoPort, int ServoAngleInDegrees)
        {
            //Create command string
            String sServo = "servo "+ ServoPort.ToString() + ":" + ServoAngleInDegrees.ToString() + "\r";
            byte[] boutServo = new byte[sServo.Length];
            boutServo = Encoding.UTF8.GetBytes(sServo);

            //Send Command
            COMPort.Write(boutServo, 0, boutServo.Length);
            Thread.Sleep(300);

            //dummy read to clear buffer of ACK
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
        }

        public void RobotStop()
        {
            //Create the Stop Command String
            String sStop = "stop\r";
            byte[] boutStop = new byte[sStop.Length];
            boutStop = Encoding.UTF8.GetBytes(sStop);

            //send command and clear ACK
            COMPort.Write(boutStop, 0, boutStop.Length);
            Thread.Sleep(50);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            
        }



        //-------------------------------------------------------------------------------------------
        // Converts Strings to an Integer values
        // Used for both IR and Temp Readings
        // Assumption: number field in byte string has ASCII space, 0x20, after it: 
        // <preampble characters><ASCII digits><ASCII space><trailer characters>
        // Number is always positive
        //-------------------------------------------------------------------------------------------

        protected static int StringToInt(byte[] AsciiString)
        {
            int iReturnVal = 0;
            int idx;
            int iPwrTen;   //Conversion multiplier

            for (idx = 0; AsciiString[idx] != 0x20; idx++)
            {
                ;   //Do nothing
            }

            idx--;  //Back up to units digit
            for (iPwrTen = 1; ((AsciiString[idx] >= 0x30) && (AsciiString[idx] <= 0x39)); idx--)
            {
                iReturnVal += (AsciiString[idx] & 0x0F) * iPwrTen;  //Mask off upper ASCII nibble and convert to power of ten
                iPwrTen *= 10;
            }

            return iReturnVal;
        }



    }
}
