﻿//
//TraxsterII
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//


using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.Threading;
using Microsoft.SPOT.Hardware.SJJ;
using Microsoft.SPOT.Hardware.SJJ.SPI_LCD_DRIVER;
using System.IO.Ports;
using SJJEMS.RobotSerializer;

namespace TraxsterII
{
    public class Program
    {
        public static void Main()
        {
            Debug.Print(Resources.GetString(Resources.StringResources.String1));
            App myApp = new App();
            myApp.Run();
        }

    }

    public class App
    {
        //configure the SPI port, the LCDDriver, the Serial port and the Green LED port.
        static SPI.Configuration mySPIPortSettings = new SPI.Configuration(Pins.GPIO_NONE, false, 0, 0, false, true, 50, SPI.SPI_module.SPI1);
        SPI mySPIPort = new SPI(mySPIPortSettings);
        SPI_LCDDriver myLCDDriver = new SPI_LCDDriver();

        RobotSerializer myRobot = new RobotSerializer();
        
        OutputPort myGreenLED = new OutputPort(Pins.GREEN_LED, false);
      
        
        public int ReverseCheck = 0;
        public int sensor_Senstivity = 185;

        

        public void Run()
        {

            myRobot.RobotInitialize("COM2");

            //Ready to go!
            mySPIPort.Write(myLCDDriver.LCDClear());
            Thread.Sleep(6);
            mySPIPort.Write(myLCDDriver.LCDLine1("Ready"));
            Thread.Sleep(6);

            //This is the main While-loop that keeps the robot moving forward
            while (true)
            {
                myRobot.RobotMove(10, 10);
                mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Left"));
                myRobot.RobotServo(1, -45);

                if (SensorIR())
                {
                    Obstruction();
                }

                myRobot.RobotMove(10, 10);
                mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Center"));
                myRobot.RobotServo(1, 0);
                if (SensorIR())
                {
                    Obstruction();
                }

                myRobot.RobotMove(10, 10);
                mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Right"));
                myRobot.RobotServo(1, 45);
                if (SensorIR())
                {
                    Obstruction();
                }
                Thread.Sleep(50);

            }
        }

        //An obstruction was found, stop the motors and scan to find
        // a clear path and keep moving
        public void Obstruction()
        {

            myGreenLED.Write(true);
            mySPIPort.Write(myLCDDriver.LCDLine1("Obstruction Found"));
            Thread.Sleep(6);
            myRobot.RobotStop();
            Scan();
            Thread.Sleep(1000);
            myGreenLED.Write(false);
            mySPIPort.Write(myLCDDriver.LCDClear());
            Thread.Sleep(6);
            mySPIPort.Write(myLCDDriver.LCDLine1("Moving"));
            Thread.Sleep(6);

        }

        //Read the IR sensor. If the resulting value is greater
        //then the sensitivity value return true otherwise return false
        public bool SensorIR()
        {
            int finalNum = myRobot.RobotSensor(0);

            mySPIPort.Write(myLCDDriver.LCDClear());
            Thread.Sleep(6);
            mySPIPort.Write(myLCDDriver.LCDLine1("IR Value: " + finalNum.ToString()));

            //The sensitivity value can be changed for better sensitivity.
            if (finalNum > 135)
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        //The Scan() method determines which direction the robot should turn.
        //3 IR readings are made - left, center, right. Based on the readings
        //The robot will make a turn to avoid the obstruction.
        public void Scan()
        {

            bool ScanLeft = false;
            bool ScanCenter = false;
            bool ScanRight = false;

            TempRead();

            mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Left"));
            myRobot.RobotServo(1, -45);
            if (SensorIR())
            {
                ScanLeft = true;
            }

            mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Center"));
            myRobot.RobotServo(1, 0);
            if (SensorIR())
            {
                ScanCenter = true;
            }


            mySPIPort.Write(myLCDDriver.LCDLine2("Scanning: Right"));
            myRobot.RobotServo(1, 45);
            if (SensorIR())
            {
                ScanRight = true;
            }

            if (ScanLeft == true && ScanCenter == true && ScanRight == true)
            {
                //Obstruction ahead - turn around
                myRobot.RobotMove(-10, -10);
                Thread.Sleep(500);
                myRobot.RobotStop();
                ReverseCheck += 1;
                Reversechecking();
                Scan();
            }
            else if (ScanLeft == true && ScanCenter == true && ScanRight == false)
            {
                //turn right
                myRobot.RobotMove(-12, 12); 
                Thread.Sleep(1000);
                myRobot.RobotStop();
            }
            else if (ScanLeft == true && ScanCenter == false && ScanRight == true)
            {
                //Obstruction ahead - reverse
                myRobot.RobotMove(-10, -10);
                Thread.Sleep(500);
                myRobot.RobotStop();
                ReverseCheck += 1;
                Reversechecking();
                Scan();
            }
            else if (ScanLeft == false && ScanCenter == true && ScanRight == true)
            {
                //Turn Left
                myRobot.RobotMove(12, -12);
                Thread.Sleep(1000);
                myRobot.RobotStop();
            }
            else if (ScanLeft == false && ScanCenter == false && ScanRight == true)
            {
                //Turn Left
                myRobot.RobotMove(12, -12);
                Thread.Sleep(500);
                myRobot.RobotStop();
            }
            else if (ScanLeft == true && ScanCenter == false && ScanRight == false)
            {
                //turn right
                myRobot.RobotMove(-12, 12);
                Thread.Sleep(500);
                myRobot.RobotStop();
            }
            else if (ScanLeft == false && ScanCenter == false && ScanRight == false)
            {
                myRobot.RobotMove(10, 10);
                Thread.Sleep(500);
                //myRobot.RobotStop();
            }
            else if (ScanLeft == false && ScanCenter == true && ScanRight == false)
            {
                myRobot.RobotMove(-10, -10);
                Thread.Sleep(500);
                myRobot.RobotStop();
                ReverseCheck += 1;
                Reversechecking();
                Scan();
            }
        }


        //Sends Temperature Reading to the LCD display
        public void TempRead()
        {
            //myRobot.RobotSensor(4);

            int finalNum = myRobot.RobotSensor(4);

            float tempC = ((((float)finalNum * 5) / 1024) * 100) - 273;
            float tempF = (tempC * (float)1.8) + 32;

            mySPIPort.Write(myLCDDriver.LCDLine1("Temp: " + tempF.ToString() + "F / " + tempC.ToString() + "C"));

            Thread.Sleep(1000);
        }


        //If we backed up more than once and spin around to avoid
        //being stuck. The global ReverseCheck int is used
        public void Reversechecking()
        {
            if (ReverseCheck >= 2)
            {
                //Spin around right
                myRobot.RobotMove(-12, 12);
                Thread.Sleep(1800);
                myRobot.RobotStop();
                ReverseCheck = 0;
            }
        }
    }  

}
