﻿//
//Parallax GPS
//SJJ Embedded Micro Solutions
//Copyright (c)  2011-2012 SJJ Micro Solutions, LLC. All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

// Code assumes wire connection RX/TX and Open Collector solution from the book

using System;
using System.Text;
using System.IO.Ports;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;


namespace SJJEMS.ParallaxGPS
{
    public class ParallaxGPS
    {
        protected static SerialPort COMPort;
        protected static byte[] binCOMPortRead = new byte[15];

        public void ParallaxGPSInitialize(String SerializerPort)
        {
            COMPort = new SerialPort(SerializerPort, (int)BaudRate.Baudrate4800, Parity.None, 8);
            COMPort.ReadTimeout = 10;
            COMPort.Open();
        }

        public void GPSCommandSend(byte[] sGet)
        {
            string sGPScommand = "!GPS";
            byte[] bGPScommand = new byte[sGPScommand.Length];
            UTF8Encoding Encoding = new UTF8Encoding();
            bGPScommand = Encoding.GetBytes(sGPScommand);

            byte[] bFinalCommand = new byte[bGPScommand.Length + sGet.Length];
            bGPScommand.CopyTo(bFinalCommand, 0);
            sGet.CopyTo(bFinalCommand, bGPScommand.Length);

            COMPort.Write(bFinalCommand, 0, bFinalCommand.Length);
        }

        public string GetFirmwareVersion()
        {
            byte[] bGetInfo = new byte[] { 0x00 };
            //byte[] bGetInfoResult = new byte[2];

            GPSCommandSend(bGetInfo);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);

            int FW_MAJOR = binCOMPortRead[6] / 10;
            int FW_MINOR = binCOMPortRead[6] % 10;

            string finalValue = FW_MAJOR.ToString() + "." + FW_MINOR.ToString();
            return finalValue;
        }

        public string GetHardwareVersion()
        {
            byte[] bGetInfo = new byte[] { 0x00 };
            //byte[] bGetInfoResult = new byte[2];

            GPSCommandSend(bGetInfo);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);

            int HW_MAJOR = binCOMPortRead[5] / 10;
            int HW_MINOR = binCOMPortRead[5] % 10;

            string finalValue = HW_MAJOR.ToString() + "." + HW_MINOR.ToString();
            return finalValue;
        }

        public bool GetValid()
        {
            byte[] bGetValid = new byte[] { 0x01 };
            
            GPSCommandSend(bGetValid);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);

            if (binCOMPortRead[5] == 0)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        public int GetSats()
        {
            byte[] bGetSats = new byte[] { 0x02 };
            
            GPSCommandSend(bGetSats);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);

            return binCOMPortRead[5];            
        }

        public string GetTime()
        {
            byte[] bGetTime = new byte[] { 0x03 };
            byte[] bGetTimeResult  = new byte[3];

            GPSCommandSend(bGetTime);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetTimeResult[0] = binCOMPortRead[5];
            bGetTimeResult[1] = binCOMPortRead[6];
            bGetTimeResult[2] = binCOMPortRead[7];

            return bGetTimeResult[0].ToString() + ":" + bGetTimeResult[1].ToString() + ":" + bGetTimeResult[2].ToString();
        }

        public string GetDate()
        {
            byte[] bGetDate = new byte[] { 0x04 };
            byte[] bGetDateResult = new byte[3];
            GPSCommandSend(bGetDate);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetDateResult[0] = binCOMPortRead[5];
            bGetDateResult[1] = binCOMPortRead[6];
            bGetDateResult[2] = binCOMPortRead[7];

            return bGetDateResult[0].ToString() + " / " + bGetDateResult[1].ToString() + " / " + bGetDateResult[2].ToString();
        }

        public string GetLat()
        {
            byte[] bGetLat = new byte[] { 0x05 };
            byte[] bGetLatResult = new byte[5];
            
            GPSCommandSend(bGetLat);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetLatResult[0] = binCOMPortRead[5];
            bGetLatResult[1] = binCOMPortRead[6];
            bGetLatResult[2] = binCOMPortRead[7];
            bGetLatResult[3] = binCOMPortRead[8];
            bGetLatResult[4] = binCOMPortRead[9];

            Int32 LatSeconds = ((bGetLatResult[2] * 100) + bGetLatResult[3]);
            Int32 LatSeconds_DEC = LatSeconds / 10;
            Int32 LatSeconds_MOD = LatSeconds % 10;
            string LatDir = "";

            if (bGetLatResult[4] == 0)
            {
                LatDir = "N";
            }
            else
            {
                LatDir = "S";
            }

            return bGetLatResult[0].ToString() + "° " + bGetLatResult[1].ToString() + "' " + LatSeconds_DEC.ToString() + "." + LatSeconds_MOD.ToString() + "s " + LatDir;
        }

        public float GetLatDecimal()
        {

            byte[] bGetLat = new byte[] { 0x05 };
            byte[] bGetLatResult = new byte[5];

            GPSCommandSend(bGetLat);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetLatResult[0] = binCOMPortRead[5];
            bGetLatResult[1] = binCOMPortRead[6];
            bGetLatResult[2] = binCOMPortRead[7];
            bGetLatResult[3] = binCOMPortRead[8];
            bGetLatResult[4] = binCOMPortRead[9];

            Int32 LatSeconds = ((bGetLatResult[2] * 100) + bGetLatResult[3]);
            Int32 LatMinutes = bGetLatResult[1];
            Int32 LatDegrees = bGetLatResult[0];

            float finalLatDecimalValue = (float)LatDegrees + ((float)LatMinutes / 60) + ((float)LatSeconds / 60000);
                      

            if (bGetLatResult[4] == 1)
            {
                finalLatDecimalValue *= -1;
            }
            return finalLatDecimalValue;
        }

        public string GetLong()
        {

            byte[] bGetLong = new byte[] { 0x06 };
            byte[] bGetLongResult = new byte[5];
            GPSCommandSend(bGetLong);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetLongResult[0] = binCOMPortRead[5];
            bGetLongResult[1] = binCOMPortRead[6];
            bGetLongResult[2] = binCOMPortRead[7];
            bGetLongResult[3] = binCOMPortRead[8];
            bGetLongResult[4] = binCOMPortRead[9];
            Int32 LongSeconds = ((bGetLongResult[2] * 100) + bGetLongResult[3]);
            Int32 LongSeconds_DEC = LongSeconds / 10;
            Int32 LongSeconds_MOD = LongSeconds % 10;
            string LongDir = "";

            if (bGetLongResult[4] == 0)
            {
                LongDir = "E";
            }
            else
            {
                LongDir = "W";
            }         
            
            return bGetLongResult[0].ToString() + "° " + bGetLongResult[1].ToString() + "' " + LongSeconds_DEC.ToString() + "." + LongSeconds_MOD.ToString() + "s " + LongDir;

        }

        public float GetLongDecimal()
        {
            byte[] bGetLong = new byte[] { 0x06 };
            byte[] bGetLongResult = new byte[5];
            GPSCommandSend(bGetLong);
            Thread.Sleep(1500);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetLongResult[0] = binCOMPortRead[5];
            bGetLongResult[1] = binCOMPortRead[6];
            bGetLongResult[2] = binCOMPortRead[7];
            bGetLongResult[3] = binCOMPortRead[8];
            bGetLongResult[4] = binCOMPortRead[9];

            Int32 LongSeconds = ((bGetLongResult[2] * 100) + bGetLongResult[3]);
            Int32 LongMinutes = bGetLongResult[1];
            Int32 LongDegrees = bGetLongResult[0];

            float finalLongDecimalValue = (float)LongDegrees + ((float)LongMinutes / 60) + ((float)LongSeconds / 60000);
            if(bGetLongResult[4] == 1)
            {

                finalLongDecimalValue *= -1;
            }

            return finalLongDecimalValue;
        }

        public string GetAlt()
        {
            byte[] bGetAlt = new byte[] { 0x07 };
            byte[] bGetAltResult = new byte[2];
            GPSCommandSend(bGetAlt);
            Thread.Sleep(1000);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetAltResult[0] = binCOMPortRead[5];
            bGetAltResult[1] = binCOMPortRead[6];

            Int32 decAltitude = ((bGetAltResult[0] * 100) + bGetAltResult[1]);
            Int32 decAltitude_DEC = decAltitude / 10;
            Int32 decAltitude_MOD = decAltitude % 10;

            return decAltitude_DEC.ToString() + "." + decAltitude_MOD.ToString();
        }

        public string GetSpeed()
        {
            byte[] bGetSpeed = new byte[] { 0x08 };
            byte[] bGetSpeedResult = new byte[2];
            GPSCommandSend(bGetSpeed);
            Thread.Sleep(2000);
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetSpeedResult[0] = binCOMPortRead[5];
            bGetSpeedResult[1] = binCOMPortRead[6];

            Int32 decSpeed = ((bGetSpeedResult[0] * 100) + bGetSpeedResult[1]);
            Int32 decSpeed_DEC = decSpeed / 10;
            Int32 decSpeed_MOD = decSpeed % 10;

            return decSpeed_DEC.ToString() + "." + decSpeed_MOD.ToString();
        }

        public string GetHeading()
        {
            byte[] bGetHeading = new byte[] { 0x09 };
            byte[] bGetHeadingResult = new byte[2];
            COMPort.Read(binCOMPortRead, 0, binCOMPortRead.Length);
            bGetHeadingResult[0] = binCOMPortRead[5];
            bGetHeadingResult[1] = binCOMPortRead[6];

            Int32 decHeading = ((bGetHeadingResult[0] * 100) + bGetHeadingResult[1]);
            Int32 decHeading_DEC = decHeading / 10;
            Int32 decHeading_MOD = decHeading % 10;

            return decHeading_DEC.ToString() + "." + decHeading_MOD.ToString();
        }

    }
}
