﻿//SPI_LCD_4x20_Driver
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//


using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware.SJJ.SPI_LCD_DRIVER;
using Microsoft.SPOT.Hardware;

namespace Microsoft.SPOT.Hardware.SJJ.SPI_LCD_4x20_Extension_Driver
{
    public class SPI_LCD_4x20_Driver : SPI_LCDDriver
    {
        // Override the line length of the display
        protected static new int ilineLength = 20;

        private const int iLineIndex = 2;
        protected byte[] bSetCursorPreamble = new byte[] { 0xF5, 0x8A };
        protected byte[] bWriteAtCursorPreamble = new byte[] { 0xF5, 0x80 };

        public byte[] LCD4LLineNoWrap(string sDisplayString, byte bLine)
        {
            // Construct the display lines
            //Size the array to manage write preamble, 1 blank char for selector indicator, and actual string
            byte[] bWriteMessageLine = new byte[bWriteLCDLine1Preamble.Length + sDisplayString.Length + 1];

            //Truncate string to size of display
            string sTrimString = "";
            for (int i = 0; (i < ilineLength) && (i < sDisplayString.Length); i++)
            {
                sTrimString += sDisplayString[i];
            }

            switch (bLine)
            {
                case 1:
                    {
                        bWriteMessageLine = base.LCDLine1(sTrimString);
                        break;
                    }
                case 2:
                    {
                        bWriteMessageLine = base.LCDLine2(sTrimString);
                        break;
                    }
                case 3:
                case 4:
                    {
                        bWriteMessageLine = base.LCDLine1(sTrimString);
                        // Overwrite line index
                        bWriteMessageLine[iLineIndex] = bLine;
                        break;
                    }
                default:
                    {
                        bWriteMessageLine = base.LCDLine1NoWrap("LCD Line Index Error");
                        break;
                    }
            }

            return bWriteMessageLine;
        }

        public byte[] WriteByteAtCursor(byte bChar)
        {
            //Write a single byte to the current cursor location
            byte[] bWriteMsg = new byte[bWriteAtCursorPreamble.Length + 1];
            bWriteAtCursorPreamble.CopyTo(bWriteMsg, 0);
            bWriteMsg[bWriteAtCursorPreamble.Length] = bChar;
            return bWriteMsg;
        }

        public byte[] WriteStringAtCursor(string sMsg)
        {
            // Convert string literals to byte arrays
            byte[] bMsg = new byte[sMsg.Length];

            // convert literal unicode strings to byte arrays
            System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
            bMsg = Encoding.GetBytes(sMsg);

            byte[] bWriteMsg = new byte[bWriteAtCursorPreamble.Length + sMsg.Length];
            bWriteAtCursorPreamble.CopyTo(bWriteMsg, 0);
            bMsg.CopyTo(bWriteMsg, bWriteAtCursorPreamble.Length);

            return bWriteMsg;
        }

        public byte[] WriteByteChar(byte bChar, byte bDisplayLine, byte bColumn)
        {
            //Write a one-byte character to any line or column - Note: no range checking
            byte[] bWriteSelectorMsg = new byte[bWriteLCDLine1Preamble.Length + 1];
            bWriteLCDLine1Preamble.CopyTo(bWriteSelectorMsg, 0);
            bWriteSelectorMsg[bWriteSelectorMsg.Length - 1] = bChar;
            bWriteSelectorMsg[2] = bDisplayLine;
            bWriteSelectorMsg[3] = bColumn;
            return bWriteSelectorMsg;
        }
        public byte[] WriteByteChar(byte bChar, byte bDisplayLine)
        {
            //Overload: Write a one-byte character to the first column of any line
            byte[] bWriteSelectorMsg = new byte[bWriteLCDLine1Preamble.Length + 1];
            bWriteLCDLine1Preamble.CopyTo(bWriteSelectorMsg, 0);
            bWriteSelectorMsg[bWriteSelectorMsg.Length - 1] = bChar;
            bWriteSelectorMsg[2] = bDisplayLine;
            return bWriteSelectorMsg;
        }

        public byte[] SetCursor(byte bLine, byte bColumn)
        {
            byte[] bWriteCursorMsg = new byte[bSetCursorPreamble.Length + 2];
            bSetCursorPreamble.CopyTo(bWriteCursorMsg, 0);
            bWriteCursorMsg[bSetCursorPreamble.Length] = bLine;
            bWriteCursorMsg[bSetCursorPreamble.Length + 1] = bColumn;
            return bWriteCursorMsg;
        }

    }
}
