﻿//FullSystem_Test
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using System.Threading;

using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

using GHIElectronics.NETMF.FEZ;

using SJJEMS.RobotSerializer;
using SJJEMS.ParallaxGPS;
using Microsoft.SPOT.Hardware.SJJ.SPI_LCD_DRIVER;
using Microsoft.SPOT.Hardware.SJJ.SPI_LCD_4x20_Extension_Driver;


namespace Full_System_Test
{
    public class Program
    {
        public static void Main()
        {
            App myApp = new App();
            myApp.Run();
        }
    }

    public class App
    {

        RobotSerializer myRobot = new RobotSerializer();
        OutputPort myLED = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.LED, false);
        public int ReverseCheck = 0;
        public int sensor_Senstivity = 200;
        public static SPI mySPIPort;
        public static SPI_LCD_4x20_Driver mySPILCD = new SPI_LCD_4x20_Driver();
        ParallaxGPS myGPS = new ParallaxGPS();
        public static OutputPort LED1 = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.IO22, false);
        public static OutputPort LED2 = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.IO21, false);
        public static OutputPort LED3 = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.IO20, false);

        public void Run()
        {
            SPI.Configuration mySPIPortConfig = new SPI.Configuration((Cpu.Pin)FEZ_Pin.Digital.IO50, false, 0, 0, false, true, 50, SPI.SPI_module.SPI1);
            mySPIPort = new SPI(mySPIPortConfig);
            myGPS.ParallaxGPSInitialize("COM2");           
            myRobot.RobotInitialize("COM1");


            SPIWriteLoop(mySPILCD.LCDClear());
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCDBacklightOn());
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCDHomeCursor());
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCDBlinkblockCursorOff());
            Thread.Sleep(6);

            //Wait until the GPS has acquired satellites before going.            
            while (!(myGPS.GetValid()))
            {
                SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Looking for Sats", 1));
                Thread.Sleep(6);
                SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Sats acquired: " + myGPS.GetSats().ToString(), 2));
                Thread.Sleep(6);
                SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Valid Data: " + myGPS.GetValid().ToString(), 3));
                Thread.Sleep(6);
            }
             

            //start moving
            while (true)
            {
                myRobot.RobotMove(10, 10);
                Scan();
                Thread.Sleep(100);
            }
        }

        public void Scan()
        {
            bool ScanLeft = false;
            bool ScanCenter = false;
            bool ScanRight = false;

            if (myRobot.RobotSensor(0) > sensor_Senstivity)
            {
                ScanLeft = true;
                myRobot.RobotStop();
                DisplayLocalStats();
                myLED.Write(true);
            }
            if (myRobot.RobotSensor(1) > sensor_Senstivity)
            {
                ScanCenter = true;
                myRobot.RobotStop();
                DisplayLocalStats();
                myLED.Write(true);
            }
            if (myRobot.RobotSensor(2) > sensor_Senstivity)
            {
                ScanRight = true;
                myRobot.RobotStop();
                DisplayLocalStats();
                myLED.Write(true);
            }

            if (ScanLeft == true && ScanCenter == true && ScanRight == true)
            {
                //Obstruction ahead - turn around
                myRobot.RobotMove(-10, -10);
                Thread.Sleep(500);
                myRobot.RobotStop();
                LED1.Write(true);
                LED2.Write(true);
                LED3.Write(true);
                ReverseCheck += 1;
                Reversechecking();
                Scan();
            }
            else if (ScanLeft == true && ScanCenter == true && ScanRight == false)
            {
                //turn right
                myRobot.RobotMove(-12, 12);
                Thread.Sleep(1000);
                myRobot.RobotStop();
                LED1.Write(false);
                LED2.Write(false);
                LED3.Write(true);
            }
            else if (ScanLeft == true && ScanCenter == false && ScanRight == true)
            {
                //Obstruction ahead - turn around
                myRobot.RobotMove(-10, -10);
                Thread.Sleep(500);
                myRobot.RobotStop();
                LED1.Write(true);
                LED2.Write(false);
                LED3.Write(true);
                ReverseCheck += 1;
                Reversechecking();
                Scan();
            }
            else if (ScanLeft == false && ScanCenter == true && ScanRight == true)
            {
                //Turn Left
                myRobot.RobotMove(12, -12);
                Thread.Sleep(1000);
                myRobot.RobotStop();
                LED1.Write(false);
                LED2.Write(true);
                LED3.Write(true);
            }
            else if (ScanLeft == false && ScanCenter == false && ScanRight == true)
            {
                //Turn Left
                myRobot.RobotMove(12, -12);
                Thread.Sleep(500);
                myRobot.RobotStop();
                LED1.Write(false);
                LED2.Write(false);
                LED3.Write(true);
            }
            else if (ScanLeft == true && ScanCenter == false && ScanRight == false)
            {
                //turn right
                myRobot.RobotMove(-12, 12);
                Thread.Sleep(500);
                myRobot.RobotStop();
                LED1.Write(true);
                LED2.Write(false);
                LED3.Write(false);
            }
            else if (ScanLeft == false && ScanCenter == false && ScanRight == false)
            {
                myRobot.RobotMove(10, 10);
                Thread.Sleep(500);
                LED1.Write(false);
                LED2.Write(false);
                LED3.Write(false);
            }
            else if (ScanLeft == false && ScanCenter == true && ScanRight == false)
            {
                //Obstruction ahead - turn around
                myRobot.RobotMove(-10, -10);
                Thread.Sleep(500);
                myRobot.RobotStop();
                LED1.Write(false);
                LED2.Write(true);
                LED3.Write(false);
                ReverseCheck += 1;
                Reversechecking();
                Scan();
            }

            myLED.Write(false);
        }

        public void Reversechecking()
        {
            if (ReverseCheck >= 2)
            {
                //Spin around Right
                myRobot.RobotMove(-12, 12);
                Thread.Sleep(1800);
                myRobot.RobotStop();
                ReverseCheck = 0;
            }
        }

        public void DisplayLocalStats()
        {

            int tempValue = myRobot.RobotSensor(4);
            float tempC = ((((float)tempValue * 5) / 1024) * 100) - 273;
            float tempF = (tempC * (float)1.8) + 32;

            SPIWriteLoop(mySPILCD.LCDClear());
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCDBacklightOn());
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCDHomeCursor());
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCDBlinkblockCursorOff());
            Thread.Sleep(6);

            //SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Temp C: " + tempC.ToString(), 1));
            //Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Temp F: " + tempF.ToString(), 1));
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Long: " + myGPS.GetLong(), 2));
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Lat: " + myGPS.GetLat(), 3));
            Thread.Sleep(6);
            SPIWriteLoop(mySPILCD.LCD4LLineNoWrap("Alt: " + myGPS.GetAlt(), 4));
            Thread.Sleep(6);
        }

        public static void SPIWriteLoop(byte[] bWriteMessage)
        {

            byte[] Xfer = new byte[1];

            for (int i = 0; i < bWriteMessage.Length; i++)
            {
                Xfer[0] = bWriteMessage[i];
                mySPIPort.Write(Xfer);
            }
        }
        
    }
}
