﻿//EX702_GPS_FEZ_Panda_II
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//


using System;
using System.Threading;
using System.IO.Ports;
using System.Text;

using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;
using GHIElectronics.NETMF.IO;

namespace EX702_GPS_FEZ_Panda_II
{
    public class Program
    {
        public static void Main()
        {
            App myApp = new App();
            myApp.Run();            
        }

    }

    public class App
    {

        public OutputPort myRED_LED = new OutputPort((Cpu.Pin)FEZ_Pin.Digital.LED, false);
        public byte[] binNowArray = new byte[15];
        public SerialPort COM1port = new SerialPort(Serial.COM1, (int)BaudRate.Baudrate4800, Parity.None, 8);


        public byte[] bGetInfo = new byte[] { 0x00 };
        public byte[] bGetValid = new byte[] { 0x01 };
        public byte[] bGetSats = new byte[] { 0x02 };
        public byte[] bGetTime = new byte[] { 0x03 };
        public byte[] bGetDate = new byte[] { 0x04 };
        public byte[] bGetLat = new byte[] { 0x05 };
        public byte[] bGetLong = new byte[] { 0x06 };
        public byte[] bGetAlt = new byte[] { 0x07 };
        public byte[] bGetSpeed = new byte[] { 0x08 };
        public byte[] bGetHeading = new byte[] { 0x09 };

        public byte[] bGetInfoResult = new byte[2];
        public byte[] bGetValidResult = new byte[1];
        public byte[] bGetSatsResult = new byte[1];
        public byte[] bGetTimeResult = new byte[3];
        public byte[] bGetDateResult = new byte[3];
        public byte[] bGetLatResult = new byte[5];
        public byte[] bGetLongResult = new byte[5];
        public byte[] bGetAltResult = new byte[2];
        public byte[] bGetSpeedResult = new byte[2];
        public byte[] bGetHeadingResult = new byte[2];

        public void Run()
        {
            COM1port.ReadTimeout = 10;
            COM1port.Open();

            GPSCommandSend(bGetInfo);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);

            int HW_MAJOR = binNowArray[5] / 10;
            int HW_MINOR = binNowArray[5] % 10;
            int FW_MAJOR = binNowArray[6] / 10;
            int FW_MINOR = binNowArray[6] % 10;

            Debug.Print("Hardware Version = " + HW_MAJOR.ToString() + "." + HW_MINOR.ToString());
            Debug.Print("Firmware Version = " + FW_MAJOR.ToString() + "." + FW_MINOR.ToString());

            GPSCommandSend(bGetValid);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetValidResult[0] = binNowArray[5];
            Debug.Print("Signal Valid = " + bGetValidResult[0].ToString());


            GPSCommandSend(bGetSats);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetSatsResult[0] = binNowArray[5];
            Debug.Print("Satellites Aquired = " + bGetSatsResult[0].ToString());

            GPSCommandSend(bGetTime);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetTimeResult[0] = binNowArray[5];
            bGetTimeResult[1] = binNowArray[6];
            bGetTimeResult[2] = binNowArray[7];
            Debug.Print("UTC Time = " + bGetTimeResult[0].ToString() + ":" + bGetTimeResult[1].ToString() + ":" + bGetTimeResult[2].ToString());

            GPSCommandSend(bGetDate);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetDateResult[0] = binNowArray[5];
            bGetDateResult[1] = binNowArray[6];
            bGetDateResult[2] = binNowArray[7];
            Debug.Print("UTC Date = " + bGetDateResult[0].ToString() + " / " + bGetDateResult[1].ToString() + " / " + bGetDateResult[2].ToString());

            GPSCommandSend(bGetLat);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetLatResult[0] = binNowArray[5];
            bGetLatResult[1] = binNowArray[6];
            bGetLatResult[2] = binNowArray[7];
            bGetLatResult[3] = binNowArray[8];
            bGetLatResult[4] = binNowArray[9];

            Int32 LatSeconds = ((bGetLatResult[2] * 100) + bGetLatResult[3]);
            Int32 LatSeconds_DEC = LatSeconds / 10;
            Int32 LatSeconds_MOD = LatSeconds % 10;
            string LatDir = "";


            if (bGetLatResult[4] == 0)
            {
                LatDir = "N";
            }
            else
            {
                LatDir = "S";
            }
            Debug.Print("Latitude = " + bGetLatResult[0].ToString() + "° " + bGetLatResult[1].ToString() + "' " + LatSeconds_DEC.ToString() + "." + LatSeconds_MOD.ToString() + "s " + LatDir);


            GPSCommandSend(bGetLong);
            Thread.Sleep(1500);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetLongResult[0] = binNowArray[5];
            bGetLongResult[1] = binNowArray[6];
            bGetLongResult[2] = binNowArray[7];
            bGetLongResult[3] = binNowArray[8];
            bGetLongResult[4] = binNowArray[9];

            Int32 LongSeconds = ((bGetLongResult[2] * 100) + bGetLongResult[3]);
            Int32 LongSeconds_DEC = LongSeconds / 10;
            Int32 LongSeconds_MOD = LongSeconds % 10;
            string LongDir = "";

            if (bGetLongResult[4] == 0)
            {
                LongDir = "E";
            }
            else
            {
                LongDir = "W";
            }
            Debug.Print("Longitude = " + bGetLongResult[0].ToString() + "° " + bGetLongResult[1].ToString() + "' " + LongSeconds_DEC.ToString() + "." + LongSeconds_MOD.ToString() + "s " + LongDir);


            GPSCommandSend(bGetAlt);
            Thread.Sleep(1000);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetAltResult[0] = binNowArray[5];
            bGetAltResult[1] = binNowArray[6];

            Int32 decAltitude = ((bGetAltResult[0] * 100) + bGetAltResult[1]);
            Int32 decAltitude_DEC = decAltitude / 10;
            Int32 decAltitude_MOD = decAltitude % 10;

            Debug.Print("Altitude = " + decAltitude_DEC.ToString() + "." + decAltitude_MOD.ToString() + " meters");

            GPSCommandSend(bGetSpeed);
            Thread.Sleep(2000);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetSpeedResult[0] = binNowArray[5];
            bGetSpeedResult[1] = binNowArray[6];

            Int32 decSpeed = ((bGetSpeedResult[0] * 100) + bGetSpeedResult[1]);
            Int32 decSpeed_DEC = decSpeed / 10;
            Int32 decSpeed_MOD = decSpeed % 10;

            Debug.Print("Speed = " + decSpeed_DEC.ToString() + "." + decSpeed_MOD.ToString() + " knots");

            GPSCommandSend(bGetHeading);
            Thread.Sleep(2000);
            COM1port.Read(binNowArray, 0, binNowArray.Length);
            bGetHeadingResult[0] = binNowArray[5];
            bGetHeadingResult[1] = binNowArray[6];

            Int32 decHeading = ((bGetHeadingResult[0] * 100) + bGetHeadingResult[1]);
            Int32 decHeading_DEC = decHeading / 10;
            Int32 decHeading_MOD = decHeading % 10;

            Debug.Print("Heading = " + decHeading_DEC.ToString() + "." + decHeading_MOD.ToString() + "°");

            // Blink board LED
            bool ledState = false;
            while (true)
            {
                // Sleep for 500 milliseconds
                Thread.Sleep(500);

                // toggle LED state
                ledState = !ledState;
                myRED_LED.Write(ledState);
            }

        }

        public void GPSCommandSend(byte[] sGet)
        {
            string sGPScommand = "!GPS";
            byte[] bGPScommand = new byte[sGPScommand.Length];
            UTF8Encoding Encoding = new UTF8Encoding();
            bGPScommand = Encoding.GetBytes(sGPScommand);

            byte[] bFinalCommand = new byte[bGPScommand.Length + sGet.Length];
            bGPScommand.CopyTo(bFinalCommand, 0);
            sGet.CopyTo(bFinalCommand, bGPScommand.Length);
            
            COM1port.Write(bFinalCommand, 0, bFinalCommand.Length);
        }

    }

}
