﻿
//SPI_LCD_DRIVER
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using Microsoft.SPOT;

namespace Microsoft.SPOT.Hardware.SJJ.SPI_LCD_DRIVER
{
    public class SPI_LCDDriver
    {
        protected const int ilineLength = 16;

        //SPI Keypad Reads
        protected byte[] SPIWriteKeyPadBuf = new byte[3] { 0xF5, 0xD1, 0x00 };
        public byte[] KeyPadReadBuf = new byte[4];


        // LCD Commands
        // Turn display on/off
        protected byte[] bWriteLCDOn = new byte[] { 0xF5, 0x1A };
        protected byte[] bWriteLCDOff = new byte[] { 0xF5, 0x12 };
        // Backlight on/off
        protected byte[] bWriteLCDBacklightOn = new byte[] { 0xF5, 0x28 };
        protected byte[] bWriteLCDBacklightOff = new byte[] { 0xF5, 0x20 };
        //Cursor operations
        protected byte[] bWriteLCDHomeCursor = new byte[] { 0xF5, 0x8B };
        protected byte[] bWriteLCDBlinkblockCursorOn = new byte[] { 0xF5, 0x18 };
        protected byte[] bWriteLCDBlinkblockCursorOff = new byte[] { 0xF5, 0x10 };
        //protected byte[] bWriteLCDCusorMoveForward = new byte[] { 0xF5, 0x09 };
        //protected byte[] bWriteLCDCusorMoveBackward = new byte[] { 0xF5, 0x01 };
        protected byte[] bWriteLCDUnderlineCursorOn = new byte[] { 0xF5, 0x19 };
        protected byte[] bWriteLCDUnderlineCursorOff = new byte[] { 0xF5, 0x11 };
        protected byte[] bWriteLCDMoveCursorRight = new byte[] { 0xF5, 0x83 };
        protected byte[] bWriteLCDMoveCursorLeft = new byte[] { 0xF5, 0x84 };
        //Display operations
        protected byte[] bWriteLCDShiftDisplayRight = new byte[] { 0xF5, 0x85 };
        protected byte[] bWriteLCDShiftDisplayLeft = new byte[] { 0xF5, 0x86 };
        protected byte[] bWriteLCDShiftDisplayUp = new byte[] { 0xF5, 0x87 };
        protected byte[] bWriteLCDShiftDisplayDown = new byte[] { 0xF5, 0x88 };
        protected byte[] bWriteLCDClear = new byte[] { 0xF5, 0x8C };

        //LCD Read Status
        protected byte[] SPIWriteLCDReadStatus = new byte[3] { 0xF5, 0xD0, 0x00 };
        public byte[] ReadLCDStatusBuf = new byte[4];

        // LCD send byte string to lines preamble
        protected byte[] bWriteLCDLine1Preamble = new byte[] { 0xF5, 0x8A, 0x01, 0x01, 0xF5, 0x80 };
        protected byte[] bWriteLCDLine2Preamble = new byte[] { 0xF5, 0x8A, 0x02, 0x01, 0xF5, 0x80 };

        public byte[] LCDOn()
        {
            return (bWriteLCDOn);
        }

        public byte[] LCDOff()
        {
            return (bWriteLCDOff);
        }

        public byte[] LCDBacklightOn()
        {
            return (bWriteLCDBacklightOn);
        }

        public byte[] LCDBacklightOff()
        {
            return (bWriteLCDBacklightOff);
        }

        public byte[] LCDHomeCursor()
        {
            return (bWriteLCDHomeCursor);
        }

        public byte[] LCDBlinkblockCursorOn()
        {
            return (bWriteLCDBlinkblockCursorOn);
        }

        public byte[] LCDBlinkblockCursorOff()
        {
            return (bWriteLCDBlinkblockCursorOff);
        }

        public byte[] LCDUnderlineCursorOn()
        {
            return (bWriteLCDUnderlineCursorOn);
        }

        public byte[] LCDUnderlineCursorOf()
        {
            return (bWriteLCDUnderlineCursorOff);
        }

        public byte[] LCDMoveCursorRight()
        {
            return (bWriteLCDMoveCursorRight);
        }

        public byte[] LCDMoveCursorLeft()
        {
            return (bWriteLCDMoveCursorLeft);
        }

        public byte[] LCDClear()
        {
            return (bWriteLCDClear);
        }

        public byte[] LCDShiftDisplayRight()
        {
            return (bWriteLCDShiftDisplayRight);
        }

        public byte[] LCDShiftDisplayLeft()
        {
            return (bWriteLCDShiftDisplayLeft);
        }

        public byte[] LCDShiftDisplayUp()
        {
            return (bWriteLCDShiftDisplayUp);
        }

        public byte[] LCDShiftDisplayDown()
        {
            return (bWriteLCDShiftDisplayDown);
        }

        public byte[] LCDReadStatus()
        {
            return (SPIWriteLCDReadStatus);
        }

        public byte[] LCDKeyPadRead()
        {
            return (SPIWriteKeyPadBuf);
        }

        public byte[] LCDLine1(string sDisplayString)
        {

            // Convert string literals to byte arrays
            byte[] bDisplayString = new byte[sDisplayString.Length];

            // convert literal unicode strings to byte arrays
            System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
            bDisplayString = Encoding.GetBytes(sDisplayString);

            // Construct the display lines
            //Size the array
            byte[] bWriteMessageLine = new byte[bWriteLCDLine1Preamble.Length + bDisplayString.Length];

            //copy the preamble for line 1 first followed by the string starting at the
            //end of the pre-able (using the length)
            bWriteLCDLine1Preamble.CopyTo(bWriteMessageLine, 0);
            bDisplayString.CopyTo(bWriteMessageLine, bWriteLCDLine1Preamble.Length);

            return bWriteMessageLine;
        }

        public byte[] LCDLine2(string sDisplayString)
        {

            // Convert string literals to byte arrays
            byte[] bDisplayString = new byte[sDisplayString.Length];

            // convert literal unicode strings to byte arrays
            System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
            bDisplayString = Encoding.GetBytes(sDisplayString);

            // Construct the display lines
            //Size the array
            byte[] bWriteMessageLine = new byte[bWriteLCDLine2Preamble.Length + bDisplayString.Length];

            //copy the preamble for line 1 first followed by the string starting at the
            //end of the pre-able (using the length)
            bWriteLCDLine2Preamble.CopyTo(bWriteMessageLine, 0);
            bDisplayString.CopyTo(bWriteMessageLine, bWriteLCDLine1Preamble.Length);

            return bWriteMessageLine;
        }

        public byte[] LCDLine1NoWrap(string sDisplayString)
        {
            string sTrimString = "";
            for (int i = 0; (i < ilineLength) && (i < sDisplayString.Length); i++)
            {
                sTrimString += sDisplayString[i];
            }

            // Convert string literals to byte arrays
            byte[] bDisplayString = new byte[ilineLength];

            // convert literal unicode strings to byte arrays
            System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
            bDisplayString = Encoding.GetBytes(sTrimString);

            // Construct the display lines
            //Size the array
            byte[] bWriteMessageLine = new byte[bWriteLCDLine1Preamble.Length + bDisplayString.Length];

            //copy the preamble for line 1 first followed by the string starting at the
            //end of the pre-able (using the length)
            bWriteLCDLine1Preamble.CopyTo(bWriteMessageLine, 0);
            bDisplayString.CopyTo(bWriteMessageLine, bWriteLCDLine1Preamble.Length);

            return bWriteMessageLine;
        }

        public byte[] LCDLine2NoWrap(string sDisplayString)
        {
            string sTrimString = "";
            for (int i = 0; (i < ilineLength) && (i < sDisplayString.Length); i++)
            {
                sTrimString += sDisplayString[i];
            }

            // Convert string literals to byte arrays
            byte[] bDisplayString = new byte[sTrimString.Length];

            // convert literal unicode strings to byte arrays
            System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
            bDisplayString = Encoding.GetBytes(sTrimString);

            // Construct the display lines
            //Size the array
            byte[] bWriteMessageLine = new byte[bWriteLCDLine2Preamble.Length + bDisplayString.Length];

            //copy the preamble for line 1 first followed by the string starting at the
            //end of the pre-able (using the length)
            bWriteLCDLine2Preamble.CopyTo(bWriteMessageLine, 0);
            bDisplayString.CopyTo(bWriteMessageLine, bWriteLCDLine1Preamble.Length);

            return bWriteMessageLine;
        }
    }
}
