﻿//
//SPI_Serial2
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//


using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Hardware.SJJ;
using Microsoft.SPOT.Hardware.SJJ.SPI_Serial_Driver;
using System.Threading;

namespace SPI_Serial2
{
    public class Program
    {
        public static void Main()
        {
            Debug.Print(Resources.GetString(Resources.StringResources.String1));
            Debug.Print(Microsoft.SPOT.Hardware.SJJ.Version.HWProviderVer);
            Debug.Print(Microsoft.SPOT.Hardware.SJJ.SPI_Serial_Driver.Version.SpiSerVersion);
            App myApp = new App();
            myApp.Run();

        }

        public class App
        {
            static SPI.Configuration myMaximSPIPortConfig = new SPI.Configuration(Cpu.Pin.GPIO_NONE, false, 0, 0, false, true, 5000, SPI.SPI_module.SPI1);
            SPI mySerialSPIPort = new SPI(myMaximSPIPortConfig);

            static SPI_Serial_Driver mySpiSerial = new SPI_Serial_Driver();

            InterruptPort EGPIO2;
            OutputPort myGreenLED;

            ushort[] uiReadData = new ushort[1];
            ushort[] uiWriteData = new ushort[1];
            ushort[] uiReadStatus = new ushort[1];
            ushort[] uiWriteReadData = new ushort[1];
            bool bMaximCTSStatus = false;

            const int c_iBuffSize = 4096;
            ushort[] uiLineBuffer = new ushort[c_iBuffSize];    //circular buffer
            static int iWriteIndex = 0;
            static int iReadIndex = 0;

            public void Run()
            {
                EGPIO2 = new InterruptPort(Pins.EGPIO2_HDR3_15, false, Port.ResistorMode.Disabled, InterruptModes.InterruptEdgeLow);
                EGPIO2.OnInterrupt += new NativeEventHandler(EGPIO2_OnInterrupt);

                myGreenLED = new OutputPort(Pins.GREEN_LED, false);

                // Write serial port configuration; using default config: 115200, 8, N, 1
                mySerialSPIPort.WriteRead(mySpiSerial.SPI_Serial_Config(), uiReadData);

                // Enable incoming serial data, Clear RTS 
                mySerialSPIPort.WriteRead(mySpiSerial.ClearRTS(), uiReadStatus);

                // Check CTS status
                bMaximCTSStatus = mySpiSerial.CheckCTS(uiReadStatus[0]);

                // Character echo loop & status LED flash loop
                uint iLoopCount = 0;
                uint iValidReceiveCount = 0;
                bool bLEDState = true;
                while (true)
                {
                    // Check read buffer and echo back the characters
                    if ((iWriteIndex != iReadIndex) && bMaximCTSStatus)
                    {
                        mySerialSPIPort.WriteRead(mySpiSerial.WriteChar(uiLineBuffer[iWriteIndex]), uiWriteReadData);

                        if (mySpiSerial.CheckTxEmpty(uiWriteReadData[0]))
                        {
                            // If write buffer not full, increment the index otherwise send it again next time around
                            iWriteIndex++;
                            if (iWriteIndex >= c_iBuffSize)
                            {
                                // Wrap index
                                iWriteIndex = 0;
                            }
                        }

                        // Store any valid data read during the write
                        if (mySpiSerial.CheckDataValid(uiWriteReadData[0]))
                        {
                            uiLineBuffer[iReadIndex++] = uiWriteReadData[0];

                            // Check for read index wrap
                            if (iReadIndex >= c_iBuffSize)
                            {
                                // Wrap buffer
                                iReadIndex = 0;
                            }
                        }

                        // Enable incoming serial data, Clear RTS; check CTS 
                        mySerialSPIPort.WriteRead(mySpiSerial.ClearRTS(), uiReadStatus);
                        bMaximCTSStatus = mySpiSerial.CheckCTS(uiReadStatus[0]);
                    }

                    //Idle loop; flash green LED to show activity without using a Thread.Sleep
                    if ((iLoopCount++ % 1000) == 0)
                    {
                        if (bLEDState)
                        {
                            bLEDState = false;
                        }
                        else
                        {
                            bLEDState = true;

                            //Output the valid receive char count on WriteRead
                            if (iValidReceiveCount > 0)
                            {
                                Debug.Print(iValidReceiveCount.ToString());

                                // Reset count
                                iValidReceiveCount = 0;
                            }
                        }
                        myGreenLED.Write(bLEDState);
                    }
                }

            }

            void EGPIO2_OnInterrupt(uint data1, uint data2, DateTime time)
            {
                // Do NULL reads only in this interrupt handler; keep it lean & mean
                mySerialSPIPort.WriteRead(mySpiSerial.uiReadCommand, uiReadData);

                // Check CTS
                bMaximCTSStatus = mySpiSerial.CheckCTS(uiReadData[0]);

                //If data valid, store it in the circular buffer; keep it lean & mean
                if (mySpiSerial.CheckDataValid(uiReadData[0]))
                {
                    uiLineBuffer[iReadIndex++] = uiReadData[0];

                    // Check for read index wrap
                    if (iReadIndex >= c_iBuffSize)
                    {
                        // Wrap buffer
                        iReadIndex = 0;
                    }
                }
            }
        }
    }
}
