﻿//EX904_PWM2_EDK
//
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.Threading;
using Microsoft.SPOT.Hardware.SJJ;
using Microsoft.SPOT.Hardware.SJJ.SJJ_PWM_Driver;
using System.IO.Ports;


namespace EX904_PWM2_EDK
{
    public class Program
    {
        public static void Main()
        {
            Debug.Print(Resources.GetString(Resources.StringResources.String1));
            App myApp = new App();
            myApp.Run();
        }

        public class App
        {

            // Create PWM driver object
            SJJ_PWM_Driver myPWMDriver = new SJJ_PWM_Driver();

            // Create green LED object
            OutputPort myGreenLED;

            public void Run()
            {
                // Open the PWM device
                SerialPort myPwmHandle = myPWMDriver.PWM_Open();

                // Create green LED object for visual feedback and turn it off to start
                myGreenLED = new OutputPort(Pins.GREEN_LED, false);

                SJJ_PWM_Driver.PWM_Status eStatusPWM;
                bool bToggle = false;

                // Turn PWM1 off
                eStatusPWM = myPWMDriver.PWM_Off(myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_Off failed: " + eStatusPWM.ToString());
                }

                // Set PWM1 frequency to 500K Hz
                eStatusPWM = myPWMDriver.PWM_SetFreq(500000, myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_SetFreq failed: " + eStatusPWM.ToString());
                }

                // Set PWM1 duty cycle to 50%
                eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM1, 50, myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_SetDutyCycle ePWM1 failed: " + eStatusPWM.ToString());
                }

                // Set CPLD PWM2 duty cycle to 25%
                eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM2_CPLD, 25, myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_SetDutyCycle ePWM2_CPLD failed: " + eStatusPWM.ToString());
                }

                // Set CPLD PWM3 duty cycle to 75%
                eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM3_CPLD, 75, myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_SetDutyCycle ePWM3_CPLD failed: " + eStatusPWM.ToString());
                }

                // Turn on PWM
                eStatusPWM = myPWMDriver.PWM_On(myPwmHandle);
                if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                {
                    Debug.Print("PWM_On failed: " + eStatusPWM.ToString());
                }

                // Turn on green LED
                myGreenLED.Write(true);

                Thread.Sleep(500);

                // Loop and toggle CPLD duty cycles
                while (true)
                {
                    if (bToggle)
                    {
                        bToggle = false;

                        // Set CPLD PWM2 duty cycle to 75%
                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM2_CPLD, 75, myPwmHandle);
                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                        {
                            Debug.Print("PWM_SetDutyCycle ePWM2_CPLD failed: " + eStatusPWM.ToString());
                        }

                        // Set CPLD PWM3 duty cycle to 25%
                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM3_CPLD, 25, myPwmHandle);
                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                        {
                            Debug.Print("PWM_SetDutyCycle ePWM3_CPLD failed: " + eStatusPWM.ToString());
                        }

                        // Turn off green LED
                        myGreenLED.Write(false);

                    }
                    else
                    {
                        bToggle = true;

                        // Set CPLD PWM2 duty cycle to 25%
                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM2_CPLD, 25, myPwmHandle);
                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                        {
                            Debug.Print("PWM_SetDutyCycle ePWM2_CPLD failed: " + eStatusPWM.ToString());
                        }

                        // Set CPLD PWM3 duty cycle to 75%
                        eStatusPWM = myPWMDriver.PWM_SetDutyCycle(SJJ_PWM_Driver.PWM_Channel.ePWM3_CPLD, 75, myPwmHandle);
                        if (eStatusPWM != SJJ_PWM_Driver.PWM_Status.eStatusWriteSuccess)
                        {
                            Debug.Print("PWM_SetDutyCycle ePWM3_CPLD failed: " + eStatusPWM.ToString());
                        }

                        // Turn on green LED
                        myGreenLED.Write(true);

                    }

                    Thread.Sleep(500);
                }
            }
        }
    }
}
